@@ -9,7 +9,7 @@ import datetime
 
 from .errors import TestError
 from .targets import create_classification_target, create_regression_target
-from ..base.tabular import TabularClassificationAutoML, TabularRegressionAutoML, is_input_metatype, extract_statics
+from ..base.tabular import TabularClassificationAutoML, TabularRegressionAutoML, TabularClassifier, TabularRegressor, is_input_metatype, extract_statics
 from ..base.types import Metatype, is_classification_metatype, is_delimiter_metatype, get_delimiter
 from ..base.errors import ValidationError
 from ..metrics.metrics import get_metric
@@ -74,7 +74,19 @@ def load_tabular_data(algo, nan_fraction = 0.1):
 
     return (X, y, X_metatypes, y_metatypes)
 
-def test_tabular(algo, output_dir, timeout, nan_fraction):
+def test_tabular_algo(algo, output_dir, timeout, nan_fraction):
+    """
+    Tests tabular algo.
+    
+    Args:
+        algo: TabularClassificationAutoML or TabularRegressionAutoML.
+        output_dir: path for temp files.   
+        timeout: training time in seconds.
+        nan_fraction: fraction of missing observations in inputs.
+
+    Raises:
+        TypeError, TestError
+    """
     if not (issubclass(algo, TabularClassificationAutoML) or issubclass(algo, TabularRegressionAutoML)):
         raise TypeError('invalid algo class')
 
@@ -96,7 +108,31 @@ def test_tabular(algo, output_dir, timeout, nan_fraction):
     if fit_seconds > timeout + 10:
         raise TestError('timeout significantly exceeded')
 
-    pred = model.predict(X)
+    test_tabular_predictor(model, output_dir, X, y)
+
+def test_tabular_predictor(predictor, output_dir, X, y):
+    """
+    Tests tabular predictor.
+    
+    Args:
+        predictor: TabularClassifier or TabularRegressor.
+        output_dir: path for temp files.   
+        X: DataFrame.
+        y: DataFrame.
+
+    Raises:
+        TypeError, TestError
+    """
+    if not (isinstance(predictor, TabularClassifier) or isinstance(predictor, TabularRegressor)):
+        raise TypeError('invalid predictor')
+
+    if not isinstance(X, pd.DataFrame):
+        raise TypeError('X must be a DataFrame')
+
+    if not isinstance(y, pd.DataFrame):
+        raise TypeError('y must be a DataFrame')
+
+    pred = predictor.predict(X)
     if not isinstance(pred, pd.DataFrame):
         raise TestError('prediction must be DataFrame')
 
@@ -106,21 +142,22 @@ def test_tabular(algo, output_dir, timeout, nan_fraction):
     if list(pred.index) != list(X.index):
         raise TestError('prediction index must equal X index')
 
-    if issubclass(algo, TabularClassificationAutoML):
-        for prediction_method in statics['prediction_methods']:
+    if isinstance(predictor, TabularClassifier):
+        prediction_methods = [x.name.lower() for x in predictor.prediction_methods()]
+        for prediction_method in prediction_methods:
             if prediction_method != 'predict':
-                pred_prediction_method = getattr(model, prediction_method)(X)
-                if not algo.is_multioutput():
+                pred_prediction_method = getattr(predictor, prediction_method)(X)
+                if y.shape[1] == 1:
                     pred_prediction_method = [pred_prediction_method]
 
                 for ppm, col in zip(pred_prediction_method, y):
                     _check_prediction(prediction_method, ppm, X, y[col], pred[col])
 
     if output_dir is not None:
-        filename = os.path.join(output_dir, str(uuid.uuid4()) + '.mdl')
-        model.save(filename)
+        filename = os.path.join(output_dir, uuid.uuid4().hex + '.mdl')
+        predictor.save(filename)
 
-        model_loaded = algo.load(filename)
+        model_loaded = predictor.load(filename)
         pred_loaded = model_loaded.predict(X)
 
         if not pred_loaded.equals(pred):
@@ -128,7 +165,8 @@ def test_tabular(algo, output_dir, timeout, nan_fraction):
 
         os.remove(filename)
 
-    print(f'TESTING DONE: {type(model).__name__}')
+    print(f'TESTING DONE: {type(predictor).__name__}')
+
 
 def _check_prediction(prediction_method, ppm, X, y, pred):
     if not isinstance(ppm, pd.DataFrame):
