@@ -154,7 +154,7 @@ class BinaryTransformer(BaseEstimator, TransformerMixin):
     def __init__(self, nan_unknowns = True):
         if not isinstance(nan_unknowns, bool):
             raise TypeError('nan_unkonwns must be bool')
-        self._nan_unknowns = nan_unknowns
+        self.nan_unknowns = nan_unknowns
 
     def fit(self, X, y = None):
         """
@@ -189,21 +189,14 @@ class BinaryTransformer(BaseEstimator, TransformerMixin):
         for col, s in X.items():
             cat = self.categories[col]
             binary = s == cat[1]
-            if self._nan_unknowns:
+            if self.nan_unknowns:
                 binary[~s.isin(cat)] = np.nan
 
             transforms.append(binary)
         return pd.concat(transforms, axis=1).mask(pd.isnull(X)).astype('O')
 
-class CategoricalImputer(BaseEstimator, TransformerMixin):
-    """
-    Imputes categoricals with constant and transforms unknown as missing.
-
-    Args:
-        fill_value: value to replace nulls with.
-    """
-    def __init__(self, fill_value):
-        self._fill_value = fill_value
+class UnknownCategoryTransformer(BaseEstimator, TransformerMixin):
+    """Sets unknown categories to nan."""
 
     def fit(self, X, y = None):
         """
@@ -231,13 +224,11 @@ class CategoricalImputer(BaseEstimator, TransformerMixin):
         Returns:
             DataFrame.
         """
-        X = X.copy()
         idx_unknowns = []
         for col, s in X.items():
             idx_unknowns.append(~s.isin(self.categories[col]))
 
-        X[pd.concat(idx_unknowns, axis=1) | pd.isnull(X)] = self._fill_value
-        return X
+        return X.mask(pd.concat(idx_unknowns, axis=1))
 
 class ColumnTransformer(BaseEstimator, TransformerMixin):
     """ColumnTransformer specifically for DataFrame.
